from dataclasses import dataclass

@dataclass
class Config:
    # General
    seed: int = 42
    device: str = "cuda"

    # Training
    epochs: int = 30
    batch_size: int = 128
    learning_rate: float = 0.1
    momentum: float = 0.9
    weight_decay: float = 5e-4

    # Attack params
    epsilon: float = 8 / 255
    alpha: float = 2 / 255
    num_steps: int = 10
    random_init: bool = True

    # Defense selection
    defense_method: str = "JointTraining"

    # AIR parameters
    lambda_SD: float = 1.0
    lambda_IR: float = 1.0
    lambda_AR: float = 1.0
    lambda_Reg: float = 1.0
    alpha_range: tuple = (0.3, 0.7)
    use_rdrop: bool = True

    # Isotropic replay augmentations
    iso_noise_std: float = 0.01
    iso_clamp_min: float = 0.0
    iso_clamp_max: float = 1.0
    iso_p_flip: float = 0.5
    iso_flip_dim: int = 3
    iso_p_rotation: float = 0.5
    iso_max_rotation: int = 10
    iso_p_crop: float = 0.5
    iso_p_erase: float = 0.5

    # Dataset
    dataset: str = "MNIST"
    data_root: str = "./data"
    num_workers: int = 2

    # LFL
    lambda_lfl: float = 1.0
    feature_lambda: float = 1.0
    freeze_classifier: bool = True

    # JointTraining
    joint_lambda: float = 0.5

    # VanillaAT
    adv_lambda: float = 1.0

    # FeatExtraction
    feat_lambda: float = 1.0
    noise_std: float = 0.01

    # EWC
    lambda_ewc: float = 100.0

    # Multi-task or multi-attack scenario
    # attack_sequence: tuple = ("None", "FGSM")
    # attack_sequence: tuple = ("FGSM", "None")
    # attack_sequence: tuple = ("None", "PGD")
    # attack_sequence: tuple = ("PGD", "None")
    attack_sequence: tuple = ("FGSM", "PGD")
    # attack_sequence: tuple = ("PGD", "FGSM")
    # attack_sequence: tuple = ("None", "FGSM", "PGD")
    # attack_sequence: tuple = ("PGD", "FGSM", "None")


