import matplotlib.pyplot as plt
import os
import torch
import numpy as np
from sklearn.manifold import TSNE


# A simple function to plot train/val loss curves.
def plot_loss_curve(train_losses, val_losses=None, save_path=None):
    plt.figure(figsize=(8, 6))
    plt.plot(train_losses, label='Train Loss', marker='o')
    if val_losses is not None:
        plt.plot(val_losses, label='Val Loss', marker='x')
    plt.xlabel('Epoch')
    plt.ylabel('Loss')
    plt.title('Training Loss Curve')
    plt.legend()
    plt.grid(True)

    if save_path is not None:
        dir_path = os.path.dirname(save_path)
        if dir_path and not os.path.exists(dir_path):
            os.makedirs(dir_path)
        plt.savefig(save_path)
    else:
        plt.show()
    plt.close()


# This function applies t-SNE to high-dimensional features
# and plots them in 2D for visualization.
def plot_tsne(features, labels, save_path=None, title="t-SNE"):
    if isinstance(features, torch.Tensor):
        features = features.cpu().numpy()
    if isinstance(labels, torch.Tensor):
        labels = labels.cpu().numpy()

    tsne = TSNE(n_components=2, init='pca', learning_rate='auto')
    feats_2d = tsne.fit_transform(features)

    plt.figure(figsize=(8, 6))
    scatter = plt.scatter(feats_2d[:, 0], feats_2d[:, 1], c=labels, cmap='tab10', alpha=0.6)
    plt.colorbar(scatter, ticks=np.unique(labels))
    plt.title(title)
    plt.grid(True)

    if save_path is not None:
        dir_path = os.path.dirname(save_path)
        if dir_path and not os.path.exists(dir_path):
            os.makedirs(dir_path)
        plt.savefig(save_path)
    else:
        plt.show()
    plt.close()
